@@ -1,12 +1,16 @@
 import binascii
-import cPickle
 from datetime import datetime
 from time import time
 
-from bits import *
+from hazelcast.serialization.bits import *
 from hazelcast.serialization.api import StreamSerializer
 from hazelcast.serialization.base import HazelcastSerializationError
 from hazelcast.serialization.serialization_const import *
+from hazelcast import six
+from hazelcast.six.moves import range, cPickle
+
+if not six.PY2:
+    long = int
 
 
 class BaseSerializer(StreamSerializer):
@@ -251,12 +255,12 @@ class BigIntegerSerializer(BaseSerializer):
 
     def write(self, out, obj):
         the_big_int = -obj-1 if obj < 0 else obj
-        end_index = -1 if type(obj) == long else None
+        end_index = -1 if (type(obj) == long and six.PY2) else None
         hex_str = hex(the_big_int)[2:end_index]
         if len(hex_str) % 2 == 1:
             prefix = '0' # "f" if obj < 0 else "0"
             hex_str = prefix + hex_str
-        num_array = bytearray(binascii.unhexlify(bytearray(hex_str)))
+        num_array = bytearray(binascii.unhexlify(bytearray(hex_str, encoding="utf-8")))
         if obj < 0:
             neg = bytearray()
             for c in num_array:
@@ -311,13 +315,13 @@ class ArrayListSerializer(BaseSerializer):
     def read(self, inp):
         size = inp.read_int()
         if size > NULL_ARRAY_LENGTH:
-            return [inp.read_object() for _ in xrange(0, size)]
+            return [inp.read_object() for _ in range(0, size)]
         return None
 
     def write(self, out, obj):
         size = NULL_ARRAY_LENGTH if obj is None else len(obj)
         out.write_int(size)
-        for i in xrange(0, size):
+        for i in range(0, size):
             out.write_object(obj[i])
 
     def get_type_id(self):
@@ -328,7 +332,7 @@ class LinkedListSerializer(BaseSerializer):
     def read(self, inp):
         size = inp.read_int()
         if size > NULL_ARRAY_LENGTH:
-            return [inp.read_object() for _ in xrange(0, size)]
+            return [inp.read_object() for _ in range(0, size)]
         return None
 
     def write(self, out, obj):
@@ -344,7 +348,7 @@ class PythonObjectSerializer(BaseSerializer):
         return cPickle.loads(str)
 
     def write(self, out, obj):
-        out.write_utf(cPickle.dumps(obj))
+        out.write_utf(cPickle.dumps(obj, 0).decode("utf-8"))
 
     def get_type_id(self):
         return PYTHON_TYPE_PICKLE
